#include "LZ77Test.h"

#include <string/String.h>
#include <stringology/compression/LZ77Compression.h>
#include <stringology/compression/LZ77Decompression.h>

#include <iostream>
#include <tuple>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( LZ77Test, "stringology" );
CPPUNIT_TEST_SUITE_REGISTRATION ( LZ77Test );

void LZ77Test::setUp ( ) {
}

void LZ77Test::tearDown ( ) {
}

void LZ77Test::testLZ77_1 ( ) {
	std::string testStr = "aabaacaaaaaababab";

	string::LinearString < char > string = string::stringFrom ( testStr );
	std::vector < std::tuple < unsigned, unsigned, char > > res = stringology::compression::LZ77Compression::compress ( string, 6, 4 );

	std::vector < std::tuple < unsigned, unsigned, char > > expectedOutput;

	expectedOutput.push_back(std::make_tuple(0, 0, 'a' ) );
	expectedOutput.push_back(std::make_tuple(1, 1, 'b' ) );
	expectedOutput.push_back(std::make_tuple(3, 2, 'c' ) );
	expectedOutput.push_back(std::make_tuple(3, 2, 'a' ) );
	expectedOutput.push_back(std::make_tuple(1, 3, 'b' ) );
	expectedOutput.push_back(std::make_tuple(2, 3, 'b' ) );

	for ( const std::tuple < unsigned, unsigned, char > & resItem : res )
		std::cout << std::get < 0 > ( resItem ) << " " << std::get < 1 > ( resItem ) << " " << std::get < 2 > ( resItem ) << std::endl;

	CPPUNIT_ASSERT ( res == expectedOutput );

}

void LZ77Test::testLZ77_2 ( ) {

	std::string testStr = "aacaacabcabaaac";

	string::LinearString < char > string = string::stringFrom ( testStr );
	std::vector < std::tuple < unsigned, unsigned, char > > res = stringology::compression::LZ77Compression::compress ( string, 6, 4 );

	std::vector < std::tuple < unsigned, unsigned, char > > expectedOutput;

	expectedOutput.push_back(std::make_tuple(0, 0, 'a' ) );
	expectedOutput.push_back(std::make_tuple(1, 1, 'c' ) );
	expectedOutput.push_back(std::make_tuple(3, 4, 'b' ) );
	expectedOutput.push_back(std::make_tuple(3, 3, 'a' ) );
	expectedOutput.push_back(std::make_tuple(1, 2, 'c' ) );

	for ( const std::tuple < unsigned, unsigned, char > & resItem : res )
		std::cout << std::get < 0 > ( resItem ) << " " << std::get < 1 > ( resItem ) << " " << std::get < 2 > ( resItem ) << std::endl;

	CPPUNIT_ASSERT ( res == expectedOutput );
}

void LZ77Test::testLZ77Decompress1 ( ) {

	std::vector < std::tuple < unsigned, unsigned, char > > input;

	input.push_back(std::make_tuple(0, 0, 'a' ) );
	input.push_back(std::make_tuple(1, 1, 'b' ) );
	input.push_back(std::make_tuple(3, 2, 'c' ) );
	input.push_back(std::make_tuple(3, 2, 'a' ) );
	input.push_back(std::make_tuple(1, 3, 'b' ) );
	input.push_back(std::make_tuple(2, 3, 'b' ) );

	string::LinearString < char > res = stringology::compression::LZ77Decompression::decompress ( input );

	std::string string = "aabaacaaaaaababab";
	string::LinearString < char > expectedString = string::stringFrom ( string );

	CPPUNIT_ASSERT ( res == expectedString );

}

void LZ77Test::testLZ77Decompress2 ( ) {

	std::vector < std::tuple < unsigned, unsigned, char > > input;

	input.push_back(std::make_tuple(0, 0, 'a' ) );
	input.push_back(std::make_tuple(1, 1, 'c' ) );
	input.push_back(std::make_tuple(3, 4, 'b' ) );
	input.push_back(std::make_tuple(3, 3, 'a' ) );
	input.push_back(std::make_tuple(1, 2, 'c' ) );

	string::LinearString < char > res = stringology::compression::LZ77Decompression::decompress ( input );

	std::string string = "aacaacabcabaaac";
	string::LinearString < char > expectedString = string::stringFrom ( string );

	CPPUNIT_ASSERT ( res == expectedString );

}

void LZ77Test::testLZ77Decompress3 ( ) {

	std::string string = "";
	string::LinearString < char > input = string::stringFrom ( string );

	std::vector < std::tuple < unsigned, unsigned, char > > out = stringology::compression::LZ77Compression::compress ( input, 6, 4 );

	string::LinearString < char > res = stringology::compression::LZ77Decompression::decompress ( out );

	CPPUNIT_ASSERT ( input == res );
}
