#include "HeapsTest.h"

#include <alib/cpp_heap>
#include <alib/binomial_heap>
#include <alib/fibonacci_heap>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( HeapsTest, "bits" );
CPPUNIT_TEST_SUITE_REGISTRATION( HeapsTest );

void HeapsTest::setUp() {
}

void HeapsTest::tearDown() {
}

int comparator ( const int & a, const int & b ) {
	return ( b < a ) - ( a < b );
}

int less ( const int & a, const int & b ) {
	return a < b;
}

void HeapsTest::testHeaps ( ) {
	alib::BinomialHeap < int > bHeap ( comparator );
	alib::CppHeap < int > cHeap ( less );
	alib::FibonacciHeap < int > fHeap ( comparator );

	auto size = [ & ] ( ) {
		int b = bHeap.size ( );
		int c = cHeap.size ( );
		int f = fHeap.size ( );
		CPPUNIT_ASSERT ( b == c && c == f );
		return b;
	};

	auto push = [ & ] ( int a ) {
		bHeap.insert ( a );
		cHeap.insert ( a );
		fHeap.insert ( a );
	};

	auto pop = [ & ] ( ) {
		int b = bHeap.extractMax ( );
		int c = cHeap.extractMax ( );
		int f = fHeap.extractMax ( );
		if ( b != c || c != f) {
			std::cerr << "??" << std::endl << "size = " << size ( ) << std::endl;
			std::cerr << "b = " << b << " c = " << c << " f = " << f << std::endl;
		}
		CPPUNIT_ASSERT ( b == c );
		CPPUNIT_ASSERT ( c == f );
		return b;
	};

	auto mergeRandom = [ & ] ( unsigned limit ) {
		alib::BinomialHeap < int > rbHeap ( comparator );
		alib::CppHeap < int > rcHeap ( comparator );
		alib::FibonacciHeap < int > rfHeap ( comparator );

		for ( unsigned i = 0; i < limit; ++ i ) {
			unsigned val = rand ( ) % 1000;
			rbHeap.insert ( val );
			rcHeap.insert ( val );
			rfHeap.insert ( val );
		}

		bHeap.mergeWith ( std::move ( rbHeap ) );
		cHeap.mergeWith ( std::move ( rcHeap ) );
		fHeap.mergeWith ( std::move ( rfHeap ) );
	};

	push ( 1 );
	CPPUNIT_ASSERT ( size ( ) == 1 );
	CPPUNIT_ASSERT ( pop ( ) == 1 );
	std::cout << "merge random 50" << std::endl;
	mergeRandom ( 50 );
	CPPUNIT_ASSERT ( size ( ) == 50 );
	std::cout << "pop 25" << std::endl;
	for ( unsigned i = 0; i < 25; ++ i ) {
		std::cout << "pop = " << pop ( ) << std::endl;
	}
	std::cout << "merge random 50" << std::endl;
	mergeRandom ( 50 );
	CPPUNIT_ASSERT ( size ( ) == 75 );
	std::cout << "pop 25" << std::endl;
	for ( unsigned i = 0; i < 25; ++ i ) {
		std::cout << "pop = " << pop ( ) << std::endl;
	}
	std::cout << "merge random 50" << std::endl;
	mergeRandom ( 50 );
	CPPUNIT_ASSERT ( size ( ) == 100 );
	std::cout << "pop 25" << std::endl;
	for ( unsigned i = 0; i < 25; ++ i ) {
		std::cout << "pop = " << pop ( ) << std::endl;
	}
	std::cout << "merge random 50" << std::endl;
	mergeRandom ( 50 );
	CPPUNIT_ASSERT ( size ( ) == 125 );
	std::cout << "pop 25" << std::endl;
	for ( unsigned i = 0; i < 25; ++ i ) {
		std::cout << "pop = " << pop ( ) << std::endl;
	}
	std::cout << "merge random 50" << std::endl;
	mergeRandom ( 50 );
	CPPUNIT_ASSERT ( size ( ) == 150 );
	std::cout << "push random 25" << std::endl;
//	fHeap.checkConsystency ( );
	for ( unsigned i = 0; i < 25; ++ i ) {
		push ( rand ( ) % 1000 );
	}
	std::cout << "pop all" << std::endl;
	for ( unsigned i = 0; i < 175; ++ i ) {
		std::cout << "pop = " << pop ( ) << std::endl;
	}
}
